PROGRAM airqual7                                         ! Start of source code
! This program reads air quality data from a CEP format output file and 
! conforms it according to population data for later use in HAPEM.
!
! Program developed by ICF Consulting for EPA in June 2000.
! Latest revision by MLG on June 1, 2000.
! Revision by MH on March 29, 2002
! Revision for implementing the random selection from distribution data
! by MH on May 30, 2002.
! by MH on June 5,2002
! revised for state/county/census indexing mechanisms by MH on June 20,2002
! revised for variable number of air source categories by MH on May 20,2005
! Expanded m parameter to hold 80000 records by CC on August, 27,2014
! Added GETARG function that allows for parameters file to be called from command line by CC on August 27,2014
! Airqual7 finalized by ICF September, 25, 2014

!
! Files used
! 10 : param_file.2
!      PARAM file used for input constants & filenames
!      (also used by all other subprograms)
! 13 : benzout.new.txt
!      Airquality input ASCII data for the entire nation (used only here)
! 18 : log.base96.txt.usa
!      LOG file (also used by all other programs in the APPEND mode)
! 23 : benzout.da
!      Sequential airquality file created in this program and to be used in hapem
! 24 : census.da
!      Direct access population file created by INDEXPOP subprogram
! 35 : census.pop_air_da
!      Direct access file for population data of tracts matched with airtracts
!      Used by HAPEM
! 43 : benzout.air_da
!      Sequential file for air tracts matched with pop tracts and indices ranges
!      for air records in file 23. Used by HAPEM
! 44 : census_direct.ind
!      Direct access census tractID file generated by INDEXPOP
! 60 : mistract.dat
!      ASCII file showing population tracts not matched by home tracts
!      Only for book-keeping. Also used in AIRQUAL and HAEM subprograms
! 91 : benzout.state_air2_fip_range
!      Direct access file for indices of # of airquality tracts per state
!      This file is used in HAPEM subprogram
! 92 : benzout.state_air1_fip_range
!      Direct access file for indices of # of airquality counties per state
!      This file is used in HAPEM subprogram
! 93 : benzout.state_air_fip_range
!      Direct access file for indices of # of airquality tracts per county
!      This file is used in HAPEM subprogram
! 98 : statefip.dat
!      ASCII file for two digits state FIPs with their serial numbers
!      Also used by INDEXPOP and AIRQUAL subprograms
! 99 : counter.dat (This ASCII file is used in all subprograms)
!      This file has total number of chad entries in the input and output files
!
!  Declarations section
!
IMPLICIT NONE                                    ! Declare all variables
INTEGER, PARAMETER :: maxst = 53                 ! max 3 of states
INTEGER, PARAMETER :: m=80000                    ! Max # of air tracts
INTEGER, PARAMETER :: dmaxi=500						 ! Max # of data sets in an airtract
INTEGER, ALLOCATABLE :: pop1(:)			         ! Population of dem. groups
INTEGER, ALLOCATABLE :: anum(:)                  ! total # of airtracts/county
INTEGER, ALLOCATABLE :: aused(:)                 ! Indices of record numbers
INTEGER, ALLOCATABLE :: first(:), last(:)		 ! Indices of record numbers
INTEGER :: hblock, nsource, ngroup,ihblock       ! # time blocks and groups
INTEGER :: ISP1P2 = 0						     ! Indicates if the right parameter is used
INTEGER :: blockrep                              ! Repetition factors
INTEGER :: commut=1                              ! Commuting flag (1=NO, 2=YES)
INTEGER :: cl1, cl2, cr, cm                      ! Job timing variables
INTEGER :: ios,ios2,cnum,varint                       ! Read/write status indicator
INTEGER :: nair(0:m)=0,num(0:m)=0      ! Index of record numbers
INTEGER :: h,i,j,k,n,kk,istart=0,natracts=0,ist  ! Indices and counters
INTEGER :: x,y,z,idum=-1  					     ! Indices and counters
INTEGER :: first1, last1
INTEGER :: maxn=0,nfips=0,atracts=0,keep=0,atract,tractNo,arecord=0 ! Incides and Counters
INTEGER :: stsuma,stsump,ii,jj,ij,ival,iv,ntract ! Indices and counters
INTEGER :: stsumat,stsumpt                       ! Indices and counters
INTEGER :: stnum(maxst),sumcount(maxst),acount
INTEGER :: sumtr(maxst)
INTEGER :: region1,region2                       ! Serial # of states be included
INTEGER :: noinct, first11, last11
INTEGER :: ndstate
INTEGER, ALLOCATABLE :: begdist(:),enddist(:)

INTEGER :: first50,last50,ik


REAL    :: arand
REAL, ALLOCATABLE   :: aqin(:,:,:),aqout(:,:,:),aqinM(:,:,:),backC(:)    ! Pollutant conc
CHARACTER (LEN=1)   :: cf                             ! Commuting status buffer
CHARACTER (LEN=2)   :: stind(maxst)                   ! statefip
CHARACTER (LEN=3)   :: inchar                         ! Buffer for CEP block #
CHARACTER (LEN=5)   :: fir, sec                       ! county codes
CHARACTER (LEN=5)   :: county1,county2                ! county codes
CHARACTER (LEN=5)   :: fips(3500)='00000',afips,cfips ! State/county FIPS
CHARACTER (LEN=6)   :: sec1                           ! airtract 6 string code
CHARACTER (LEN=11)   :: TractID1,tractid2			  ! tract id in the air quality file
CHARACTER (LEN=11)  :: ctractid,atractid1,atractid2   ! Census Tract IDs
CHARACTER (LEN=100) :: f10='',f13,f18,f23,f24,f35,f43,f44  ! File names
CHARACTER (LEN=100) :: f20,f60,f91,f92,f93,f98,f99            ! file names
CHARACTER (LEN=100) :: f201,f202
CHARACTER (LEN=120) :: paramline,varname,varchar,FNAME      ! PARAMS file input
CHARACTER (LEN=999) :: inl,inl2,aqdat,aqdat2          ! Line from CEP file
CHARACTER (LEN=11), ALLOCATABLE  :: atractid(:),stractid(:)     ! Airtract IDs
CHARACTER (LEN=2), ALLOCATABLE :: stateidd(:)

CHARACTER (LEN=11) :: htract,distract
!INTEGER :: idsel,find14,nfind,findx,find15
INTEGER :: idsel,findx
!REAL :: dispro(3)
REAL,allocatable :: dispro(:,:),areapro(:)

!
!  PARAMS section
!
CALL SYSTEM_CLOCK(cl1,cr,cm)                          ! Execution start time
CALL GETARG(1,FNAME)
OPEN (UNIT=10,FILE=FNAME,RECL=120,ACTION='READ')    ! Open PARAMS file
!
DO                                                    ! Loop over records
   READ(10,'(A120)',IOSTAT=ios) paramline             ! Read one PARAM line
   IF (ios/=0) EXIT                                   ! Exit on end of file
   i =INDEX(paramline,'=')                            ! Check for equals sign
   IF (i==0) CYCLE                                    ! Discard line if no '='
   varname=ADJUSTL(paramline(1:i-1))                  ! Part on left of '='
   varchar=ADJUSTL(paramline(i+1:))                   ! Part on right of '='
   IF (INDEX(varname,'poll')>0) CYCLE                 ! Skip pollutant name
   IF (INDEX(varname,'unit')>0) CYCLE                 ! Skip pollutant units
   IF (INDEX(varname,'comm')>0) cf=varchar(1:1)       ! Read commuting flag 
   IF ((cf=='Y').OR.(cf=='y')) commut=2               ! Set commuting variable
   IF (INDEX(varname,'file')>0) THEN                  ! For file names...
      IF (INDEX(varname,'log')>0)      f18 =varchar   ! Set log file name
      IF (INDEX(varname,'quality')>0)  f13 =varchar   ! Set air quality file
      IF (INDEX(varname,'populat')>0)  f44 =varchar   ! Set population file
      IF (INDEX(varname,'mistract')>0)  f60=varchar   ! Set mistract file name
      IF (INDEX(varname,'statefip')>0)  f98=varchar   ! Set statefip file name
      IF (INDEX(varname,'counter')>0)   f99=varchar   ! Set counter file name 
	  IF (INDEX(varname,'DistToRoad')>0)   f20=varchar
   ELSE                                               ! Else set parameters
      READ(varchar,*,IOSTAT=ios) varint               ! Convert to integer
      IF (ios/=0) CYCLE                               ! If not integer, discard
      IF (varname=='hblock')  hblock =varint          ! Set # time blocks/day
      IF (varname=='nsource') nsource=varint          ! Set # seasons per year
      IF (varname=='ngroup')  ngroup =varint         ! Set # demographic groups
      IF (varname=='region1')        region1=varint   ! first state for run
      IF (varname=='region2')        region2=varint   ! last state for rn
	  IF (varname=='nreplic')  ISP1P2=1				  ! check if the right parameter file is used
   END IF                                             ! Finished with line
END DO                                                ! Get next line
CLOSE(10)                                             ! Close PARAMS file
!
commut=2	! block the non-commuting option 9/30/2005

IF (ISP1P2 == 0) then
	write(*,*) 'Wrong parameter file.  Use P2.txt for this module'
	stop
END IF 
!  Setup section
!
i=INDEX(f44,'.')-1                                    ! Locate extension on f44
IF (i==0) i=LEN_TRIM(f44)                             ! Trim f44 file name
f24=f44(:i)//'.da'                                    ! Index file name (input)
f44=f44(:i)//'_direct.ind'                            ! Index file name (input)
i=INDEX(f13,'.')-1                                    ! Locate extension on f13
IF (i==0) i=LEN_TRIM(f13)                             ! Trim f13 file name
f23=f13(:i)//'.da'                                    ! Direct access file name
f43=f13(:i)//'.air_da'                                ! Direct access file name
f91=f13(:i)//'.state_air2_fip_range'                  ! Direct access file name
f92=f13(:i)//'.state_air1_fip_range'                  ! Direct access file name
f93=f13(:i)//'.state_air_fip_range'                   ! Direct access file name
f35=f13(:i)//'.pop_air_da'                            ! Index file name (input)
!
i=INDEX(f20,'.')-1
IF (i==0) i=LEN_TRIM(f20)
f201=f20(:i)//'.STIDX'
f202=f20(:i)//'.dat'


Allocate(dispro(3,ngroup),areapro(3))

OPEN (13,FILE=f13,RECL=999,ACTION='READ')             ! Open CEP file
!OPEN (14,FILE='ProcessedAirData.txt', ACTION='WRITE') ! QA File
OPEN (18,FILE=f18,POSITION='REWIND',ACTION='WRITE')   ! Open log file
OPEN (23,FILE=f23,ACTION='WRITE')   ! Open AQ output file
OPEN (24,FILE=f24,ACCESS='DIRECT',RECL=4*ngroup)      ! population input file
!OPEN (35,FILE=f35,ACCESS='DIRECT',RECL=4*ngroup)      ! population output  file
OPEN (35,FILE=f35,ACTION='WRITE')
!OPEN (43,FILE=f43,ACCESS='DIRECT',RECL=8*3)   ! Open AQ output file
OPEN (431,FILE='temp3.txt',status='scratch',ACTION='readwrite')
OPEN (43,FILE=f43,ACTION='READWRITE')
OPEN (44,FILE=f44,ACCESS='direct', recl = 30)         ! census tractid
OPEN (60,FILE=f60, POSITION='rewind',action='write')  ! mistracts file
!OPEN (91,FILE=f91,ACCESS='DIRECT', RECL=50)           ! # of tracts/state
!OPEN (92,FILE=f92,ACCESS='DIRECT', RECL=50)           ! # of counties/state
!OPEN (93,FILE=f93,ACCESS='DIRECT', RECL=50)           ! # of airtracts/county
OPEN (98,FILE=f98,status='unknown')                   ! statefip file
OPEN (99,FILE=f99,status='old')                       ! counter file
OPEN (93,FILE=f93, ACTION='write')
OPEN (92,FILE=f92, ACTION='write')
OPEN (91,FILE=f91, ACTION='write')
OPEN (201,FILE=f201,ACTION='read')
OPEN (202,FILE=f202,ACCESS='direct',RECL=15+3*ngroup*4,ACTION='read')

WRITE(18,*) 'Airqual Start time= ',cl1,cr,cm          ! Execution start time

!
231 format (I14,200E14.6)
!read state fips in two digits
!
ndstate=0
DO 

read(201,*,iostat=ios)
if(ios/=0) exit
ndstate=ndstate+1

END DO

Allocate(stateidd(ndstate),begdist(ndstate), enddist(ndstate))

rewind(201)
Do i=1, ndstate
read (201,*) k,stateidd(i), begdist(i), enddist(i)

end do 


do ii = 1, maxst
  read (98, 81) ij, stind(ii)
end do
81 format (i2, 6x, a, 6x, i2)
!
!find total  number of records in airquality file for memory allocation
!
atract = 0
do
  READ (13,'(A999)',IOSTAT=ios)
  IF (ios/=0) EXIT                                               
  atract = atract +1
end do
rewind (13)

!
ALLOCATE (aqout(hblock,nsource,dmaxi))                     ! Dynamic array for output
ALLOCATE (pop1(ngroup))  
ALLOCATE (aused(atract),first(atract),last(atract),atractid(atract),stractid(atract))
!Initialize
aqout=0                                              

aused = 0
aqin=0.0
!
!  Air quality data section


!
DO                                                 ! loop for airquality tracts
  ival = 0
  READ (13,'(A999)',IOSTAT=ios) inl                              
  IF (ios/=0) EXIT                                 ! end of file 
  IF ((inl(1:6)=='Source').AND.(INDEX(inl,'ALL')>0))  keep=1 ! match with source  
  IF ((inl(1:6)=='Source').AND.(INDEX(inl,'ALL')==0)) keep=0     
  IF (keep==0) CYCLE                                             
  i=INDEX(inl,'block',back=.true.)                               
! find number of hour blocks for airquality data
  IF (i>0) THEN                                                  
    inchar=inl(i+5:i+7)                                         
    READ(inchar,*) ihblock
    IF (ihblock==hblock) THEN
      blockrep = 1
    else IF (ihblock<hblock) THEN
      IF (MOD(hblock,ihblock)/=0) THEN
        WRITE(18,*) 'Incompatible number of blocks'
        STOP
      END IF
      blockrep=hblock/ihblock
    else if (ihblock>hblock) THEN
      write (18,*) 'number of time blocks in airqual is more than required'
      stop
    END IF
    IF (.NOT.ALLOCATED(aqin)) ALLOCATE(aqin(ihblock,nsource,2), aqinM(ihblock,nsource,dmaxi), backC(dmaxi))   
  END IF 
  IF (inl(4:4)>='0'.AND.inl(4:4)<='9') THEN
    do ist = region1, region2                            ! state loop
      if (inl(4:5) == stind(ist)) then
        ival = 1
        exit
      end if
    end do                                               ! end of state loop
    if (ival == 1) then

	  atracts=atracts+1
	  atractid(atracts)=inl(4:8)//inl(11:16)                      
      DO i=1,11                                                   
         IF (atractid(atracts)(i:i)==' ') atractid(atracts)(i:i)='0' 
      END DO  

	  first1=last1 + 1
	  tractID1 = atractid(atracts)
	  aqdat=inl(17:)
      read(aqdat,*) backC(1), aqinM(:,:,1)
	  tractNo = 1
	  Do												 ! Check if there are multiple datasets
			READ (13,'(A999)',IOSTAT=ios2) inl2			 ! in the same tract.  If YES, the dataset
			if (ios2 /=0) EXIT 
			tractid2=inl2(4:8)//inl2(11:16)                      
			DO i=1,11                                                   
				IF (tractid2(i:i)==' ') tractid2(i:i)='0' 
			END DO
			if (tractid2 == TractID1) then			 ! a data set
				tractNo= tractNo + 1					 
				aqdat2 = inl2(17:)
				read(aqdat2,*) backC(tractNo),aqinM(:,:,tractNo)	
			else 
				backspace(13)
				exit
			end if 
	  End do
   	  last1 = last1 + tractNo                                           
	  first(atracts)=first1
	  last(atracts)=last1

! Expand the 3-hr airquality data based on activity time interval(if needed)
	
	DO x=1, tractNo
      DO h=1, ihblock
        DO i=1,blockrep
          DO j=1, nsource
            aqout(i+blockrep*(h-1),j,x)=aqinM(h,j,x)
          END DO
        END DO
      END DO
	  arecord = arecord + 1
!	  write(*,*) arecord,x
	  WRITE(23,231,IOSTAT=ios) arecord, aqout(:,:,x),backC(x)
!	  WRITE(14,141) atractid(atracts),aqout(:,:,x),x
	END DO
	
	  
!	  141 format (A11,32ES17.8,i6)            
! Find number of counties
      afips=atractid(atracts)(1:5)                                
      IF (COUNT(fips(1:nfips)==afips)==0) THEN                    
        nfips=nfips+1                                            
        fips(nfips)=afips                                        
      END IF                                                      
    end if										  ! end of if(ival==1)
  END IF                                          ! end of if(inl(4:4)>='0'..               
END DO                                            ! end of airtract loop
!
CLOSE(13) 
!CLOSE(14)

                                                      

WRITE(18,*) '# air tracts =', atracts, '     # of air records =', arecord                      
WRITE(18,*) '# counties on air file =', nfips                     
!

!  Indexing section
!
write (60,*) 'MISSING TRACTS for AIRQUAL & POPULATION DATA in AIRQUAL'
!
! Read number of population tracts from counter file
!
read (99,*)
read (99,*) ntract
!
!
DO kk = 1, ntract                                       ! population tract loop
  READ(44,rec = kk) ctractid,cnum                            
  ival = 0
  iv = 0
  do ist = region1, region2                             ! state loop
    if (stind(ist) == ctractid(1:2)) then               ! find the right state
      ival = 1
      exit
    end if
  end do                                                ! end of state loop
  if (ival == 1) then
    istart = istart+1
    READ(24,rec=cnum) pop1                            ! read census data
    cfips=ctractid(1:5)                                            
    n=0                                                            
                                                 
    DO j=1,atracts                                      ! air tract loop
	  IF (atractid(j)=='       ') cycle
      IF (atractid(j)==ctractid) THEN                             
        n=n+1 
        iv = 1
        natracts = natracts+1
!		write (*,431) natracts 
        aused(j)=1                                               
        write (35,*,IOSTAT=ios) pop1                ! write only matched population data
        WRITE (431,*,IOSTAT=ios) atractid(j),first(j),last(j)     ! write only matched air tracts and indices ranges
        atractid(j) = '       '
        exit
      END IF                                                      
    END DO                                              ! end of airtract loop
    num(n)=num(n)+1                                                
    IF (maxn<n) maxn=n                                             
    IF (COUNT(fips(1:nfips)==cfips)>0) nair(n)=nair(n)+1           
    IF (n>1) n=1                                                   
  end if
  if (iv == 0) write (60,104) cnum, ctractid
END DO                                               ! end of census tract loop
!
104 format (1x,i8,1x,a)
431 format ('# of the air tracts matched with pop tracts',I7)
write (*,431) natracts
REWIND(431)
!
! Read airtractsids for those which matched with population tracts
!
!
first11=0
last11=0
acount=0
sumcount=0
sumtr=0

Do

	read (431, *,IOSTAT=ios) atractid1
	if(ios/=0) exit
	county1=atractid1(1:5)
	noinct=1
	acount=acount+1
	first11 =last11+1

	do 

		read(431,*,IOSTAT=ios) atractid2
		if(ios/=0) exit
		county2=atractid2(1:5)
		if(county2==county1) then
			noinct=noinct+1
		else 
			backspace(431)
			exit
		end if
	end do 

	last11=last11+noinct
	write(93,*,IOSTAT=ios) county1,first11,last11

	do jj=1, maxst
		if(county1(1:2)== stind(jj)) then
			sumcount(jj)=sumcount(jj)+1
			sumtr(jj)=sumtr(jj)+noinct
			exit
		end if
	end do 		

end do



stsuma = 0
stsump = 0
stsumpt = 0
stsumat = 0
do ii = 1, maxst
   stsuma = stsump + sumcount(ii)
   stsumat = stsumpt + sumtr(ii)
!   write (92,rec=ii) stind(ii), stsump+1, stsuma
   write (92,*) stind(ii), stsump+1, stsuma
!   write (91,rec=ii) stind(ii), stsumpt+1, stsumat
   write (91,*) stind(ii), stsumpt+1, stsumat
!  write (55,80) ii, stind(ii), stsump+1, stsuma, sumcount(ii)
   stsump = stsuma
   stsumpt = stsumat
end do
!

!Read/Write data to  counter file
!
IF (commut == 2) then
  read (99,*)
  read (99,*)
END IF
write (99,*) natracts, acount
write (99,*) atracts,arecord
!

80 format (i2,2x,a,2x,i6,2x,i6,2x,i6)
!
! adding distance probability info

rewind(431)
4311 format (A12,2I12, 33F8.4)
DO 

	read (431,*,iostat=ios) htract,first50,last50
	if(ios/=0) exit
	idsel=0
	findx=0
	do i=1, ndstate
		if(htract(1:2) == stateidd(i)) then
		idsel=i
		exit
		end if		
	end do 
	if(idsel>0) then
	do j=begdist(idsel),enddist(idsel)
		read(202,rec=j) distract,areapro(:),(dispro(:,ik),ik=1,ngroup)
!		write(*,*) distract
!		write(*,*) htract
!		write(*,*) begdist(idsel),' ',enddist(idsel),' ',idsel,' ',j
!		stop
		if(htract==distract) then
		write(43,4311) htract,first50,last50, areapro(:),(dispro(:,ik), ik=1,ngroup)
		findx=1
		exit
		end if
	end do
	else if (idsel==0) then
		write(*,*) 'error with files'
		stop
	end if 
	if(findx==0) then
		write(*,*) 'error with files'
		stop
	end if

end do 

!  Wrapup section
!
deallocate(dispro,areapro)
CLOSE(44)                                           ! Close input indexing file
CLOSE(23)                                           ! Close AQ output file
! To log
WRITE(18,*) 'There were ',COUNT(aused(1:atracts)==0),' unpaired air tracts.'  
DO j=1,atracts                                      ! Loop over air tracts
   IF (aused(j)==0) WRITE(18,*) atractid(j)         ! Write unpaired tracts to log
END DO                                              ! Next air tract
WRITE(18,*) 'Overall, there were:'                  ! Write summary to log
WRITE(18,*) num(0), ' unpaired census tracts.'      ! Message to log
WRITE(18,*) num(1), ' census tracts with a matching air tract.'   ! Message to log
WRITE(18,*) SUM(num(2:)), ' census tracts with 2 or more air tracts.' ! Message to log
WRITE(18,*) 'Within the counties on the air file, ther were:'     ! Message to log
WRITE(18,*) nair(0), ' unpaired census tracts.'                ! Message to log
WRITE(18,*) nair(1), ' census tracts with a matching air tract.'  ! Message to log
WRITE(18,*) SUM(nair(2:)), ' census tracts with 2 or more air tracts.'! Message to log
DEALLOCATE(aqin, aqout)                               ! Release dynamic arrays
CALL SYSTEM_CLOCK(cl2,cr,cm)                          ! Get execution stop time
WRITE(18,*) 'Airqual End time = ',cl2                 ! Write time to log
WRITE(18,*) 'Airqual Job time = ',REAL(cl2-cl1)/cr    ! Total time for job
CLOSE(18)                                             ! Close log file
STOP                                                  ! Stop execution
END PROGRAM airqual7                                  ! Last line of code



