PROGRAM durav8                                       ! Start of Fortran 90 code
!
! This is the first of five programs in the HAPEM5 model. This program reads 
! CHAD activity pattern data as processed by two SAS programs (one for CHAD 
! and the other for clustering). The durations in each micro are accumulated
! over 3 hour time intervals. The output has one record per diary, grouped  
! by (agegroup/daytype/comtype) combinations.
!
! Program developed by ICF Consulting for EPA in May 2000.
! Latest revision by MLG on May 23, 2000.
! Revised by MH and AR in August 2001
! Revised by MH and AR in May 2002
! e for removing oversized arrays and random-access files by MH on June 18,2002
! Modified by MH for cluster mapping in August,2002
! Upgraded to durav6 in September 2005
! Added GETARG function to obtain parameters filename from command line by CC on August 27,2014
! Durav7 finalized by CC and ICF on February, 19, 2015
! August 2023 modifications by WGG at ICF: 
!   DURAV reads two activity input files:
!     unit 11 = CHAD diary database with personal data and hourly durations in micro
!     unit 12 = clustering results, indexed by chadid.
!   Unlike DURAV7, this version requires that units 11 and 12 have the same 
!   number of records with the same chadids in the same order. These files 
!   are produced by two SAS programs (chadhw.sas and cluster.sas).
!   It is easier to sort datasets in SAS, so do it there rather than in DURAV.
!   The commute status (comtype) is part of the new (2020 version) of CHAD, so 
!   DURAV does not need to determine the commuting status anymore.
!   With these changes, the new version of DURAV is simpler than earlier ones.    
!
! Files used
! 10 : p1.txt
!      PARAM file used for input constants & filenames, also used by other programs.
! 11 : durhw.txt
!      Activity input data read from text file, used only in DURAV.
! 12 : cluster.txt 
!      Text file with cluster classifications of chad diaries, used only by DURAV.
! 18 : log.txt
!      LOG file for whole HAPEM run, used by all other programs in the APPEND mode.
! 21 : durhw.da (Final output file used by HAPEM program)
!      Sequential file with activity data sorted by (group, daytype, comtype).
! 30 : cluster.nonzero (File used by HAPEM program)
!      This file contains counts of diaries by (group, daytype, comtype, cluster).
! 99 : counter.dat (Text file used by other subprograms)
!      This file has total number of chad entries in the input and output files
!
! Declarations section
!
IMPLICIT NONE                                             ! Must declare all variables
INTEGER, PARAMETER :: maxclus=5                           ! Maximum # clusters allowed 
INTEGER, ALLOCATABLE :: ncount(:,:,:)                     ! # diaries by (group,daytype,comtype)
INTEGER, ALLOCATABLE :: clcount(:,:,:,:)                  ! # by (group,daytype,comtype,cluster) 
INTEGER, ALLOCATABLE :: durs(:,:,:)                       ! Durations for one diary day
INTEGER, ALLOCATABLE :: bdurs(:,:,:)                      ! Durations in for N hour blocks
INTEGER :: nmicro,nblock,ntype,ngroup,hblock,fact,ncom=2  ! Upper bounds for loops
INTEGER :: dgroup,daytype,comtype,clus,nclus              ! Variables from cluster file
INTEGER :: g,dt,ct,i,ij,ik,iloop,jtest,cl                 ! Counters & indices
INTEGER :: clnocount                                      ! Cluster counts
INTEGER :: ios(21)=0                                      ! Read/write status indicators
INTEGER :: cl1,cl2,cm,cr                                  ! Job timing variables
INTEGER :: varint, dursum                                 ! PARAMS file buffer and sum of durs
CHARACTER(LEN=120) :: paramline,varname,varchar,FName     ! For PARAMS file input
CHARACTER(LEN=120) :: f10,f11,f12,f18,f21,f30,f99         ! File names
CHARACTER(LEN=100) :: fmt18                               ! File format for log
CHARACTER(LEN=1)   :: cf                                  ! Y/N status indicator
CHARACTER (LEN=9)  :: chadid                              ! ID from cluster file
TYPE personinfo                                           ! Personal data type
   CHARACTER (LEN=9) :: id                                ! CHAD ID label
   CHARACTER (LEN=5) :: zip                               ! Zip code
   CHARACTER (LEN=2) :: st                                ! Weekday, state FIPS
   CHARACTER (LEN=3) :: cou                               ! County FIPS
   CHARACTER (LEN=1) :: sex, race, emp                    ! Demographic variables
   INTEGER :: year,month,day                              ! Calendar variables
   REAL    :: age                                         ! Age on diary
   INTEGER :: group, daytype, comtype                     ! Grouping variables    
END TYPE personinfo                                       ! End of personal data
TYPE (personinfo) :: pd                                   ! Info for person-day
!
! PARAMS section
!
CALL SYSTEM_CLOCK(cl1,cr,cm)                              ! Execution start time
CALL GETARG(1,FNAME)                                      ! Command line argument

OPEN (UNIT=10,FILE=FNAME,RECL=120,ACTION='READ')          ! Open PARAMS file
DO                                                        ! Loop over records
   READ(10,'(A120)',IOSTAT=ios(10)) paramline             ! Read one PARAMS line
   IF (ios(10)/=0) EXIT                                   ! Exit on end of file
   i =INDEX(paramline,'=')                                ! Check for equal sign
   IF (i==0) CYCLE                                        ! Discard line if no '='
   varname=ADJUSTL(paramline(1:i-1))                      ! Part on left of '='
   varchar=ADJUSTL(paramline(i+1:))                       ! Part on right of '='
   IF (INDEX(varname,'poll')>0) CYCLE                     ! Skip pollutant name
   IF (INDEX(varname,'unit')>0) CYCLE                     ! Skip pollutant units
   IF (INDEX(varname,'comm')>0) cf=varchar(1:1)           ! Read commuting status
   IF ((cf=='Y').OR.(cf=='y')) ncom=2                     ! Integer commute flag 
   IF (INDEX(varname,'file')>0) THEN                      ! For file names...
      IF (INDEX(varname,'log')>0)      f18=varchar        ! Set log file
      IF (INDEX(varname,'counter')>0)  f99=varchar        ! Set counter file
      IF (INDEX(varname,'activity')>0) f11=varchar        ! Set activity file
      IF (INDEX(varname,'cluster')>0)  f12=varchar        ! Set cluster file
   ELSE                                                   ! Else set parameters
      READ(varchar,*,IOSTAT=ios(1)) varint                ! Convert to integer
      IF (ios(1)/=0) CYCLE                                ! If not integer, discard
      IF (varname=='nmicro')  nmicro =varint              ! # microenvironments
      IF (varname=='nblock')  nblock =varint              ! # time blocks/day (input)
      IF (varname=='hblock')  hblock =varint              ! # time blocks/day (output) 
      IF (varname=='ntype')   ntype  =varint              ! # day types
      IF (varname=='ngroup')  ngroup =varint              ! # demographic groups
	  IF (varname=='nreplic') then				          ! Wrong file ...
		write (*,*) ' Wrong parameter file. Use P1.txt'   ! Help message
		stop                                              ! Halt execution of DURAV
	  end if                                              ! Finished 'nreplic' logic
   END IF                                                 ! Finished 'file' logic
END DO                                                    ! Get next input line
CLOSE(10)                                                 ! Close PARAMS file
!
!  Setup section
!
WRITE(fmt18,'(A1,I4,A3)') '(',ntype,'I6)'                     ! Format for log
i = INDEX(f11,'.')-1                                          ! Locate extension
IF (i==0) i=LEN_TRIM(f11)                                     ! Trim file name 
f21=f11(:i)//'.da'                                            ! Output filename
f30=f11(:i)//'.nonzero'                                       ! Output filename
WRITE(*,*) " Output file name = ", TRIM(f21)                  ! Put name on screen            

! Find the number of hour blocks in CHAD input file
if (mod(nblock,hblock) /= 0) then
  write (18,*) 'Incompatible time blocks in CHAD file', nblock, hblock
  stop
else
  fact = nblock/hblock                                        ! Hours per time block
  WRITE(*,*) "Time blocks = ", nblock, fact
end if
!
! Open input text file
OPEN (11,FILE=f11,RECL=nmicro*nblock*8+64,ACTION='READ')      ! CHAD data file
OPEN (12,FILE=f12,ACTION='READ')                              ! CHAD cluster file
OPEN (18,FILE=f18,POSITION='REWIND',RECL=120,ACTION='WRITE')  ! Open log file
OPEN (21,FILE=f21,ACTION='write')                             ! Final output file
OPEN (30,FILE=f30,ACTION='write')                             ! File of cluster counts
OPEN (99,FILE=f99,POSITION='REWIND',RECL=120,ACTION='WRITE')  ! Open counter file

WRITE(18,*,IOSTAT=ios(18)) 'Durav Start time= ',cl1,cr,cm     ! Start time
WRITE(18,*,IOSTAT=ios(18)) 'CHAD data from file=', TRIM(f11)  ! Write to log
WRITE(18,*,IOSTAT=ios(18)) 'Clustering from file=', TRIM(f12) ! Write to log
WRITE(18,*,IOSTAT=ios(18)) 'Output data on file=', TRIM(f21)  ! Write to log
!
!  Main section
!
! Find total number of CHAD diaries in the input file for memory allocation
jtest = 0                                                     ! Initialization
READ(11, *)                                                   ! Skip header line
do                                                            ! Go until end of data
  READ (11,*,IOSTAT=ios(11)) pd                               ! Read one record
  if (ios(11) /= 0) EXIT                                      ! Exit DO if finished
  jtest = jtest+1                                             ! Increment counter
end do                                                        ! Continue reading file
write(*,*) " Input file 11: # of records = ", jtest           ! Write to screen
rewind(11)                                                    ! Return to start of file

READ (11,*,IOSTAT=ios(11))                                    ! Discard header
READ (12,*,IOSTAT=ios(11))                                    ! Discard header
!
ALLOCATE (ncount(ngroup,ntype,ncom))                          ! Dynamic arrays
ALLOCATE (clcount(ngroup,ntype,ncom,maxclus))                 ! Dynamic arrays
ALLOCATE (durs(nmicro,nblock,ncom))                           ! Dynamic arrays
ALLOCATE (bdurs(nmicro,hblock,ncom))                          ! Dynamic arrays
!
ncount = 0                                                    ! Initialization
iloop = 0                                                     ! Initialization
clcount = 0                                                   ! Initialization
211  format (I7,I3,I3,I3,I3,I7,288I5)                         ! Format for main output file

DO iloop = 1, jtest                                           ! Loop over input diaries
    READ(11,*) pd, durs                                       ! Read personal data and durs
    READ(12,*) g, dt, ct, chadid, clus, nclus                 ! Read cluster data 
    if(pd%group/=g)    WRITE(*,*) " Group mismatch:   ", pd%group, g
    if(pd%daytype/=dt) WRITE(*,*) " Daytype mismatch: ", pd%daytype, dt
    if(pd%comtype/=ct) WRITE(*,*) " Comtype mismatch: ", pd%comtype, ct
    if(pd%id/=chadid)  WRITE(*,*) " CHADID mismatch:  ", pd%id, chadid
    ncount(g, dt, ct) = ncount(g, dt, ct)+1                   ! Increment combo counter
    bdurs = 0                                                 ! Initialize 3-hour data
    do ij = 0, hblock-1                                       ! Loop over blocks 
      do ik = 1, fact                                         ! Loop over hours in block
        bdurs(:,ij+1,:)=bdurs(:,ij+1,:)+durs(:,ij*fact+ik,:)  ! Aggregate durations
      end do                                                  ! Next hour    
    end do                                                    ! Next block   
    dursum = sum(bdurs(:,:,:))                                ! Total of 3-hour durations
    if(dursum/=1440) WRITE(*,*) " Bad bdurs sum ",dursum," for ", iloop, pd
    clcount(g,dt,ct,clus) = clcount(g,dt,ct,clus)+1
    WRITE(21,211,IOSTAT=ios(21)) iloop,g,dt,ct,clus,clcount(g,dt,ct,clus),bdurs
ENDDO                                                         ! Go to next input diary
iloop = iloop-1                                               ! Counter overflows by 1

WRITE(*,*) " Number of output records = ", iloop              ! Put # diaries on screen
WRITE(99,*) jtest, iloop                                      ! Write to counter file 
101 format (21(I5))
do g=1, ngroup
	do dt=1, ntype
	   do ct=1,ncom
            clnocount = 0
            if(clcount(g,dt,ct,1)>0)  clnocount = clnocount+1
            if(clcount(g,dt,ct,2)>0)  clnocount = clnocount+1
            if(clcount(g,dt,ct,3)>0)  clnocount = clnocount+1
			write (30,101) g, dt, ct, (clcount(g,dt,ct,cl),cl = 1,3), clnocount, 0
		end do 
	end do
end do
CLOSE(11)                                                     ! close chad input file
CLOSE(12)                                                     ! close cluster input file
CLOSE(21)                                                     ! close main output file
CLOSE(99)                                                     ! close counter file
!
! Wrapup section
!
DEALLOCATE (durs,bdurs)                                                    ! Free memory
WRITE(18,*,IOSTAT=ios(18)) 'Total number of person-days processed=', iloop ! Log message
WRITE(18,*,IOSTAT=ios(18)) 'Frequency table for diaries (non-commute)'     ! Log message
WRITE(18,*,IOSTAT=ios(18)) 'By demographic group (rows) & day type (cols)' ! Log message
DO g=1, ngroup
   WRITE(18,fmt18,IOSTAT=ios(18)) ncount(g,1:ntype,1)                      ! Write 'ncount' 
ENDDO
WRITE(18,*,IOSTAT=ios(18)) 'Frequency table for diaries (commute)'         ! Log message
WRITE(18,*,IOSTAT=ios(18)) 'By demographic group (rows) & day type (cols)' ! Log message
DO g=1, ngroup
   WRITE(18,fmt18,IOSTAT=ios(18)) ncount(g,1:ntype,2)                      ! Write 'ncount' 
ENDDO
CALL SYSTEM_CLOCK(cl2,cr,cm)                                            ! Clock time
WRITE(18,*,IOSTAT=ios(18)) 'Durav End time = ',cl2                      ! Put time on log
WRITE(18,*,IOSTAT=ios(18)) 'Durav Job time = ',(cl2-cl1)/REAL(cr)       ! Elapsed time
CLOSE(18)                                                               ! Close log file
IF (ios(18)>0) PRINT *, 'Error writing to log file, IOSTAT=', ios(18)   ! Log file error
!
STOP                                                                    ! Stop execution
END PROGRAM durav8                                                      ! End of code
!

